﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.VisualBasic.FileIO;
using System.IO;
using ICSharpCode.SharpZipLib.Zip;
using System.Diagnostics;
using System.Threading;

namespace UpdatezipCreator
{
    public partial class mainForm : Form
    {
        #region Variables

        private static mainForm _mainForm;
        public static mainForm Instance
        {
            get
            {
                if (_mainForm == null) _mainForm = new mainForm();
                return _mainForm;
            }
        }
        public static string EnvironmentPath = Path.GetDirectoryName(Application.ExecutablePath);
        public static string TempPath = Path.GetTempPath() + "UpdatezipCreator";
        public static string GetTempPath
        {
            get
            {
                if (!Directory.Exists(TempPath)) Directory.CreateDirectory(TempPath);
                return TempPath;
            }
        }

        #endregion

        #region Form events

        public mainForm()
        {
            InitializeComponent();
        }

        private void mainForm_Shown(object sender, EventArgs e)
        {
            cbPushUpdate.Checked = mainSettings.Default.PushToSDCard;
            rtbUpdateScript.Text = mainSettings.Default.update_script;
            cbRecoveryReboot.Checked = mainSettings.Default.RecoveryReboot;
            rbEdify.Checked = mainSettings.Default.EdifyScript;
            rbAmend.Checked = !rbEdify.Checked;
            rbUpdateBinaryBuiltIn.Checked = mainSettings.Default.BuiltInBinaryUpdate;
            rbBinaryUpdateOwn.Checked = !rbUpdateBinaryBuiltIn.Checked;
            tbOwnUpdateBianryPath.Text = mainSettings.Default.OwnUpdateBinaryScriptPath;
            tbOwnUpdateBianryPath.Enabled = !rbUpdateBinaryBuiltIn.Checked;
            bOpenBinaryUpdate.Enabled = tbOwnUpdateBianryPath.Enabled;
            rbLinuxStyle.Checked = mainSettings.Default.IsLinuxStyle;
            rbWindowsStyle.Checked = !rbLinuxStyle.Checked;
        }

        private void mainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            mainSettings.Default.PushToSDCard = cbPushUpdate.Checked;
            mainSettings.Default.RecoveryReboot = cbRecoveryReboot.Checked;
            mainSettings.Default.update_script = rtbUpdateScript.Text;
            mainSettings.Default.EdifyScript = rbEdify.Checked;
            mainSettings.Default.BuiltInBinaryUpdate = rbUpdateBinaryBuiltIn.Checked;
            mainSettings.Default.OwnUpdateBinaryScriptPath = tbOwnUpdateBianryPath.Text;
            mainSettings.Default.IsLinuxStyle = rbLinuxStyle.Checked;
            mainSettings.Default.Save();
            ADB_Access.KillADBServer();
        }

        #endregion

        #region Update-script Page

        private void bSave_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveDialog = new SaveFileDialog();
            saveDialog.CheckPathExists = true;
            saveDialog.InitialDirectory = EnvironmentPath + "\\Scripts";

            if (saveDialog.ShowDialog() == DialogResult.OK)
            {
                rtbUpdateScript.SaveFile(saveDialog.FileName, RichTextBoxStreamType.PlainText);
            }
        }

        private void bDefaultUpdateScript_Click(object sender, EventArgs e)
        {
            
            rtbUpdateScript.Clear();
            rtbUpdateScript.Text = (rbAmend.Checked) ? mainSettings.Default.DefaultAmendScript : mainSettings.Default.DefaultEdifyScript;
        }

        private void bLoadScript_Click(object sender, EventArgs e)
        {
            OpenFileDialog openDialog = new OpenFileDialog();
            openDialog.CheckFileExists = true;
            openDialog.InitialDirectory = EnvironmentPath + "\\Scripts";

            if (openDialog.ShowDialog() == DialogResult.OK)
            {
                rtbUpdateScript.LoadFile(openDialog.FileName, RichTextBoxStreamType.PlainText);
            }
        }

        private void bMakeDefualtUpdateScript_Click(object sender, EventArgs e)
        {
            if (rbAmend.Checked) mainSettings.Default.DefaultAmendScript = rtbUpdateScript.Text;
            else if (rbEdify.Checked) mainSettings.Default.DefaultEdifyScript = rtbUpdateScript.Text;

            MessageBox.Show("Current update-script maked as defualt.", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }


        private void bAddFunc_Click(object sender, EventArgs e)
        {
            insertFunctionForm _insertForm = new insertFunctionForm(EnvironmentPath + ((rbAmend.Checked) ? "\\amend_functions.xml" : "\\edify_functions.xml"));

            if (_insertForm.ShowDialog() == DialogResult.Cancel) return;

            rtbUpdateScript.Text = rtbUpdateScript.Text.Insert(rtbUpdateScript.SelectionStart, _insertForm.ReturnTheTextToInsert());
        }

        #endregion

        #region FileTabPage

        List<UpdateEntryInfo> _FileList = new List<UpdateEntryInfo>();

        public void RefreshFileList()
        {
            lvFiles.Items.Clear();

            foreach (UpdateEntryInfo item in _FileList)
            {
                ListViewItem entry = new ListViewItem(item.FileName);
                entry.SubItems.Add(item.FilePath);
                entry.SubItems.Add(item.TargetPath);

                if (File.Exists(item.FilePath))
                entry.ImageIndex = GetIconIndex(Path.GetExtension(item.FileName));
                else if (Directory.Exists(item.FilePath)) entry.ImageIndex = 7;

                lvFiles.Items.Add(entry);
            }
        }

        private void bAddFiles_Click(object sender, EventArgs e)
        {
            OpenFileDialog openDialog = new OpenFileDialog();
            openDialog.CheckFileExists = true;
            openDialog.Multiselect = true;
            openDialog.Filter = "All Files|*.*";
            openDialog.Title = "Search for files..";
            openDialog.InitialDirectory = mainSettings.Default.AddFileInitialDirectory;

            if (openDialog.ShowDialog() != System.Windows.Forms.DialogResult.OK) return;
            else 
            {
                mainSettings.Default.AddFileInitialDirectory = openDialog.FileName;
                mainSettings.Default.Save();
            }

            addForm _addForm = new addForm();

            if (_addForm.ShowDialog() == DialogResult.Cancel | String.IsNullOrEmpty(_addForm.GetThePath())) return;

            foreach (string item in openDialog.FileNames)
            {
                UpdateEntryInfo entry = new UpdateEntryInfo(Path.GetFileName(item), item, _addForm.GetThePath());
                _FileList.Add(entry);
            }

            RefreshFileList();
        }

        private void bEditFiles_Click(object sender, EventArgs e)
        {
            if (lvFiles.SelectedIndices.Count == 0) return;

            addForm _addForm = new addForm();

            if (_addForm.ShowDialog() == DialogResult.Cancel) return;

            foreach (int index in lvFiles.SelectedIndices)
            {
                _FileList[index].TargetPath = _addForm.GetThePath();
            }

            RefreshFileList();
        }

        private void bRemove_Click(object sender, EventArgs e)
        {
            foreach (int index in lvFiles.SelectedIndices) _FileList.RemoveAt(index);
            RefreshFileList();
        }

        private void bClear_Click(object sender, EventArgs e)
        {
            _FileList.Clear();
            RefreshFileList();
        }

        private void bAddDirectory_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog browseDialog = new FolderBrowserDialog();
            browseDialog.Description = "Select a directory:";
            browseDialog.SelectedPath = SpecialDirectories.Desktop;

            if (browseDialog.ShowDialog() != DialogResult.OK) return;
            
            addForm _addForm = new addForm();
            if (_addForm.ShowDialog() != DialogResult.OK) return;

            UpdateEntryInfo entry = new UpdateEntryInfo(Path.GetFileNameWithoutExtension(browseDialog.SelectedPath), browseDialog.SelectedPath, _addForm.GetThePath());
            _FileList.Add(entry);

            RefreshFileList();


        }

        #endregion

        #region Engine

        private void CompressDirectoryRecursive(string _Path, string _RecursivePath, ZipFile _zip)
        {
            string[] _Files = Directory.GetFiles(_Path);

            foreach (string item in _Files)
            {
                LoggingClass.WriteToLog("Compressing " + item + " to " + _RecursivePath + "\\" + Path.GetFileName(_Path) + "\\" + Path.GetFileName(item));
                _zip.Add(item, _RecursivePath + "\\" + Path.GetFileName(_Path) + "\\" + Path.GetFileName(item));
            }

            string[] _Directories = Directory.GetDirectories(_Path);
 
            foreach (string item in _Directories)
            {
                CompressDirectoryRecursive(_Path + "\\" + Path.GetFileName(item), _RecursivePath + "\\" + Path.GetFileName(_Path), _zip);
            }

        }

        public void SaveLinesToFileWithLinuxChar(string fileName, List<string> lines)
        {
            TextWriter txt = new StreamWriter(fileName);
            foreach (string item in lines) txt.Write(item + "\n");
            txt.Close();

        }

        private string PrepareUpdateScript()
        {
            string update_script_filename = GetTempPath + "\\update-script";

            if (rbLinuxStyle.Checked)
            {
                List<string> lines = new List<string>();
                foreach (var item in rtbUpdateScript.Lines) lines.Add(item.Replace(Environment.NewLine, ""));
                SaveLinesToFileWithLinuxChar(update_script_filename, lines);
            }
            else
            rtbUpdateScript.SaveFile(update_script_filename, RichTextBoxStreamType.PlainText);

            LoggingClass.WriteToLog("Saved script to " + update_script_filename);
            Application.DoEvents();

            return update_script_filename;
        }

        public string CompressFiles()
        {
            try
            {
                ZipFile _zipFile = ZipFile.Create(GetTempPath + "\\update_not_signed.zip");
                LoggingClass.WriteToLog("Created zip not signed archive: " + GetTempPath + "\\update_not_signed.zip");

                _zipFile.BeginUpdate();
                _zipFile.Add(PrepareUpdateScript(), @"META-INF\com\google\android\update" + ((rbEdify.Checked) ? "r":"") + "-script");

                if (rbEdify.Checked)
                {
                    if (rbBinaryUpdateOwn.Checked && File.Exists(tbOwnUpdateBianryPath.Text)) _zipFile.Add(tbOwnUpdateBianryPath.Text, @"META-INF\com\google\android\update-binary");
                    else
                    _zipFile.Add(EnvironmentPath + "\\update-binary", @"META-INF\com\google\android\update-binary");
                }

                foreach (UpdateEntryInfo entry in _FileList)
                {
                    if (File.Exists(entry.FilePath))
                    {
                        LoggingClass.WriteToLog("Compressing " + entry.FilePath + " to " + entry.GetTargetPathTranslatedToWindowsPath + "\\" + entry.FileName);
                        _zipFile.Add(entry.FilePath, entry.GetTargetPathTranslatedToWindowsPath + "\\" + entry.FileName);
                    }
                    else
                        CompressDirectoryRecursive(entry.FilePath, entry.GetTargetPathTranslatedToWindowsPath, _zipFile);
                        
                }
                _zipFile.CommitUpdate();
                _zipFile.Close();

                LoggingClass.WriteToLog("Creating not signed zip archive done");
                return GetTempPath + "\\update_not_signed.zip";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Stop", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                LoggingClass.WriteToLog("Creating not signed archive failed. " + ex.Message);
                return null;
            }
        }

        public void ExecuteJavaCommandSync(string command)
        {
            ProcessStartInfo PSI = new ProcessStartInfo("java", command);
            PSI.RedirectStandardInput = true;
            PSI.RedirectStandardOutput = true;
            PSI.RedirectStandardError = true;
            PSI.UseShellExecute = false;
            PSI.CreateNoWindow = true;
            PSI.WindowStyle = ProcessWindowStyle.Hidden;
            PSI.WorkingDirectory = mainForm.EnvironmentPath;

            Process Console = Process.Start(PSI);
            LoggingClass.WriteToLog("Signing... \nCommand: " + command + "\nOutput: " + Console.StandardOutput.ReadToEnd());
            Console.WaitForExit();

        }

        public string GetJavaVersionInformation()
        {
            try
            {
                LoggingClass.WriteToLog("Checking java version...");
                ProcessStartInfo PSI =
                    new ProcessStartInfo("java", "-version ");

                PSI.RedirectStandardOutput = true;
                PSI.RedirectStandardError = true;
                PSI.UseShellExecute = false;
                PSI.CreateNoWindow = true;
                System.Diagnostics.Process proc = new Process();
                proc.StartInfo = PSI;
                proc.Start();
                return proc.StandardError.ReadToEnd();

            }
            catch
            {
                MessageBox.Show("Error: no java found! Archive won't be signed.", "Stop", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
        }

        private int GetIconIndex(string ext)
        {
            if (!String.IsNullOrEmpty(ext))
            {
                switch (ext.ToLower())
                {
                    case ".exe":
                        return 0;
                    case ".bat":
                        return 0;
                    case ".apk":
                        return 1;
                    case ".jpg":
                        return 2;
                    case ".jpeg":
                        return 2;
                    case ".gif":
                        return 2;
                    case ".png":
                        return 2;
                    case ".bmp":
                        return 2;
                    case ".txt":
                        return 3;
                    case ".rtf":
                        return 3;
                    case ".doc":
                        return 3;
                    case ".sh":
                        return 4;
                    case ".zip":
                        return 6;
                    case ".rar":
                        return 6;
                    case ".cab":
                        return 6;
                    default:
                        return 5;
                }
            }
            else return 5;

        }

        private void EnableControls(bool Enabled)
        {
            for (int i = 0; i < this.Controls.Count; i++)
            {
                this.Controls[i].Enabled = Enabled;
            }
            this.Refresh();
        }

        #endregion

        #region Options page

        private void bOpenBinaryUpdate_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.CheckFileExists = true;
            fileDialog.Multiselect = false;

            if (fileDialog.ShowDialog() == DialogResult.OK)
            {
                tbOwnUpdateBianryPath.Text = fileDialog.FileName;
            }
        }

        private void rbBinaryUpdateOwn_CheckedChanged(object sender, EventArgs e)
        {
            tbOwnUpdateBianryPath.Enabled = rbBinaryUpdateOwn.Checked;
            bOpenBinaryUpdate.Enabled = rbBinaryUpdateOwn.Checked;
        }

        private void rbEdify_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Do you want to load a default edify script?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) rtbUpdateScript.Text = mainSettings.Default.DefaultEdifyScript;
        }

        private void rbAmend_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Do you want to load a default amend script?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes) rtbUpdateScript.Text = mainSettings.Default.DefaultAmendScript;
        }

        #endregion

        #region Drag and Drop
        private void lvFiles_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.Copy;
            else
                e.Effect = DragDropEffects.None;
        }

        private void lvFiles_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
            {
                string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);

                addForm _addForm = new addForm();
                if (_addForm.ShowDialog() != DialogResult.OK) return;

                foreach (string item in files)
                {
                    UpdateEntryInfo entry = new UpdateEntryInfo(Path.GetFileName(item), item, _addForm.GetThePath());
                    _FileList.Add(entry);
                }

                RefreshFileList();
            }

        }

        private void lvFiles_DragOver(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else e.Effect = DragDropEffects.None;
        }
        #endregion

        #region Button click

        private void bMakeZip_Click(object sender, EventArgs e)
        {
            if (_FileList.Count == 0)
            {
                MessageBox.Show("Add some files first, please.", ":(", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                return;
            }

            try
            {
                LoggingClass.WriteToLog("Starting creating of a new archive");
                EnableControls(false);
                SaveFileDialog saveDialog = new SaveFileDialog();
                saveDialog.Filter = "Zip Archive|*.zip";
                saveDialog.DefaultExt = "zip";

                if (saveDialog.ShowDialog() == DialogResult.Cancel) return;

                string updateFilePath = saveDialog.FileName;
                LoggingClass.WriteToLog("Archive filename:  "  + updateFilePath);

                string notSignedPath = CompressFiles();
                LoggingClass.WriteToLog("Not signed archive filename: "  + notSignedPath);

                if (String.IsNullOrEmpty(notSignedPath) | !File.Exists(notSignedPath))
                {
                    LoggingClass.WriteToLog("Not signed archive doesn't exists. Aborting");
                    return;
                }

                if (!String.IsNullOrEmpty(GetJavaVersionInformation())) ExecuteJavaCommandSync("-jar signapk.jar testkey.x509.pem testkey.pk8 \"" +  notSignedPath +"\" \"" + updateFilePath + "\"");
                else
                    LoggingClass.WriteToLog("No java found. Aborting");

                if (cbPushUpdate.Checked | cbRecoveryReboot.Checked)
                {
                    ADB_Access.RunADBServer();

                    if (ADB_Access.CheckIfDeviceIsConnected())
                    {
                        if (cbPushUpdate.Checked)
                        {
                            LoggingClass.WriteToLog("Pushing file to sd card...");
                            ADB_Access.PushFile(updateFilePath, "mnt/sdcard/" + Path.GetFileName(updateFilePath));
                        }

                        if (cbRecoveryReboot.Checked)
                        {
                            LoggingClass.WriteToLog("Rebooting device to recovery...");
                            ADB_Access.RebootRecovery();
                        }
                    }
                }
                LoggingClass.WriteToLog("Done");
                MessageBox.Show("File created succesfully.", "Done", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message + "\nPlease contact the developer.", "Stop", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            finally
            {
                EnableControls(true);
            }
        }

        private void bAbout_Click(object sender, EventArgs e)
        {
            AboutForm aboutFrm = new AboutForm();
            aboutFrm.ShowDialog();
        }

        private void bSignSingleZip_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDialog = new OpenFileDialog();
            fileDialog.CheckFileExists = true;
            fileDialog.Multiselect = false;
            fileDialog.DefaultExt = "zip";
            fileDialog.Filter = "Zip archive|*.zip";

            if (fileDialog.ShowDialog() != DialogResult.OK) return;

            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.DefaultExt = "zip";
            saveFileDialog.Filter = "Zip archive|*.zip";
            saveFileDialog.FileName = Path.GetDirectoryName(fileDialog.FileName) +  "\\signed_" + fileDialog.SafeFileName;

            if (saveFileDialog.ShowDialog() != DialogResult.OK) return;

            try
            {
                EnableControls(false);
                 if (!String.IsNullOrEmpty(GetJavaVersionInformation())) ExecuteJavaCommandSync("-jar signapk.jar testkey.x509.pem testkey.pk8 \"" + fileDialog.FileName + "\" \"" + saveFileDialog.FileName + "\"");
                 MessageBox.Show("File created succesfully.", "Done", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message + "\nPlease contact the developer.", "Stop", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            finally
            {
                EnableControls(true);
            }
        }

        #endregion
    }
}
